#include "sdl/SDL.h"
#include "lua51/lua.hpp"
#include <iostream>
#include <string>
#include "funcs.h"

using namespace std;

//configuration loader
//loads config into lua, retrieves values and leaves them there
SDL_Surface *load_config(lua_State *L){
    if(luaL_loadfile(L, "scripts/config.lua") || lua_pcall(L,0,0,0)){
      cerr << "cannot load configuration script!\n" << "Error: " <<lua_tostring(L, -1) << "\n";
      return 0;
    }

    //put the config values on the stack
    lua_getglobal(L, "s_fullscreen");
    lua_getglobal(L, "s_width");
    lua_getglobal(L, "s_height");
    lua_getglobal(L, "s_depth");
    lua_getglobal(L, "s_caption");
    
    //check values for proper type
    if(!lua_isnumber(L,-5)){
      cerr << "Config Error: "<< "'s_fullscreen' should be a number\n";
      return 0;
    }
    if(!lua_isnumber(L,-4)){
      cerr << "Config Error: "<< "'s_width' should be a number\n";
      return 0;
    }
    if(!lua_isnumber(L,-3)){
      cerr << "Config Error: "<< "'s_height' should be a number\n";
      return 0;
    }
    if(!lua_isnumber(L,-2)){
      cerr << "Config Error: "<< "'s_depth' should be a number\n";
      return 0;
    }
    if(!lua_isstring(L,-1)){
      cerr << "Config Error: "<< "'s_caption' should be a string\n";
      return 0;
    }
    
    //set caption and return the surface
    SDL_WM_SetCaption(lua_tostring(L,-1), lua_tostring(L, -1));
    
    SDL_Surface *temp = SDL_SetVideoMode(lua_tointeger(L, -4), lua_tointeger(L, -3), lua_tointeger(L, -2), SDL_SWSURFACE | (lua_tointeger(L,-5)? SDL_FULLSCREEN : 0));
    for(int i=0; i < 5; i++){lua_pop(L,-1);}
    return temp;
}




//the function that is run when the timer ticks
Uint32 timer_tick(Uint32 interval, void *param){
     lua_State *L = (lua_State*)param; // convert void* to lua_State*
     
     //get function onto the stack
     lua_getglobal(L, tick_func.c_str());
     
     //call the function
     if(!lua_pcall(L, 0, 0, 0)){
       cerr << "Error: " << lua_tostring(L, -1) << "\n";
     }
     return interval;
}
