//  Lua Shooter Project
//  Author: Brett Lajzer
//  Date: 5.22.2006
//  About: This is a little vertical shooter that I wrote
//    in order to learn how to use Lua in game development.
//    The way I do it here is one of the _many_ possible ways,
//    one of which must be chosen based on appropriateness to
//    one's own project. (read: this implementation is neither
//    the best nor worst way to do it, it's just the way I
//    decided to do it so no yelling if it's ugly.)

//
// NOTE: Output streams are written to files (probably b/c of SDL).
//

#include <iostream>
#include "lua51/lua.hpp" //lua is included in the lua51 directory for good reasons
#include "sdl/SDL.h"
#include "globals.h"
#include "funcs.h"

using namespace std;

int main(int argc, char *argv[]){

    //init Lua
    lua_State *L = luaL_newstate();
    luaL_openlibs(L);
    
    //init SDL
    SDL_Init(SDL_INIT_EVERYTHING);
    
    //load up configuration file and create screen surface
    screen = load_config(L);
    if(!screen){ //die
      cerr << "Error: Cannot create video surface. \n";
      SDL_Quit();
      lua_close(L);
      return 1;
    }
    
    //register functions with lua
    lua_pushcfunction(L, l_set_timer);
    lua_setglobal(L, "set_timer");
    
    lua_pushcfunction(L, l_unset_timer);
    lua_setglobal(L, "unset_timer");
    
    lua_pushcfunction(L, l_loadbmp);
    lua_setglobal(L, "loadbmp");
    
    lua_pushcfunction(L, l_loadbmp_t);
    lua_setglobal(L, "loadbmp_t");
    
    lua_pushcfunction(L, l_loadimg);
    lua_setglobal(L, "loadimg");
    
    lua_pushcfunction(L, l_flip);
    lua_setglobal(L, "update_screen");
    
    lua_pushcfunction(L, l_blit);
    lua_setglobal(L, "blit");
    
    lua_pushcfunction(L, l_blitframe);
    lua_setglobal(L, "blit_frame");
    
    lua_pushcfunction(L, l_delay);
    lua_setglobal(L, "delay");
    
    lua_pushcfunction(L, l_mouse_state);
    lua_setglobal(L, "mouse_state");
    
    lua_pushcfunction(L, l_show_cursor);
    lua_setglobal(L, "show_cursor");
    
    lua_pushcfunction(L, l_fill_screen);
    lua_setglobal(L, "fill_screen");
    
    lua_pushcfunction(L, l_delete_image);
    lua_setglobal(L, "delete_image");
    
    lua_pushcfunction(L, l_rotzoom);
    lua_setglobal(L, "rotzoom");
    
    lua_pushcfunction(L, l_initfps);
    lua_setglobal(L, "init_fps");
    
    lua_pushcfunction(L, l_setfps);
    lua_setglobal(L, "set_fps");
    
    lua_pushcfunction(L, l_getfps);
    lua_setglobal(L, "get_fps");
    
    lua_pushcfunction(L, l_delayfps);
    lua_setglobal(L, "delay_fps");

       
    //load up the main script into the lua environment
    //and make it all happen
    int error = luaL_loadfile(L, "scripts/main.lua") ||
        lua_pcall(L, 0, 0, 0);
        
    if (error) { // die a horrible death b/c we can't run without that file
       cerr << lua_tostring(L, -1) << "\n";
       lua_pop(L,1);
       
       //end it all
       SDL_Quit();
       lua_close(L);
       return 1;           
    }

    //kill sdl
    SDL_Quit();
    
    //kill lua
    lua_close(L);

    return 0;
}
