--[[
File: collision.lua
Author: Brett Lajzer
Date: 5.25.2006

About:
Functions for collision-checking.
--]]


--checks if two objects are colliding
--based on the .x .y .h .w vars
--i realize this is really simple collision detection, but
--this project doesn't need pixel-perfect collision...
function collision(a, b)

  if a.exploding or b.exploding then return false end

  local x1, x2, y1, y2, x3, x4, y3, y4 = a.x, a.x + a.w, a.y, a.y + a.h, b.x, b.x + b.w, b.y, b.y + b.h

  
  --fully contained
  if ((x1 >= x3 and x2 <= x4)  and (y1 >= y3 and y2 <= y4)) or
    ((x3 >= x1 and x4 <= x2)  and (y3 >= y1 and y4 <= y2)) then
    return true
  end

  --top left corner
  if ((x1 >= x3 and x1 <= x4) and (y1 >= y3 and y1 <= y4)) or
    ((x3 >= x1 and x3 <= x2) and (y3 >= y1 and y3 <= y2)) then
    return true
  end

  --top right corner
  if ((x2 >= x3 and x2 <= x4) and (y1 >= y3 and y1 <= y4)) or
    ((x4 >= x1 and x4 <= x2) and (y3 >= y1 and y3 <= y2)) then
    return true
  end

  --bottom left corner
  if ((x1 >= x3 and x1 <= x4) and (y2 >= y3 and y2 <= y4)) or
    ((x3 >= x1 and x3 <= x2) and (y4 >= y1 and y4 <= y2)) then
    return true
  end

  --bottom right corner
  if ((x1 >= x3 and x1 <= x4) and (y2 >= y3 and y2 <= y4)) or
    ((x3 >= x1 and x3 <= x2) and (y4 >= y1 and y4 <= y2)) then
    return true
  end

  --top
  if ((x1 <= x3 and x2 >= x4) and (y1 >= y3 and y1 <= y4)) or
    ((x3 <= x1 and x4 >= x2) and (y3 >= y1 and y3 <= y2)) then
    return true
  end

  --left
  if ((x1 <= x3 and x1 >= x4) and (y1 >= y3 and y2 <= y4)) or
    ((x3 <= x1 and x3 >= x2) and (y3 >= y1 and y4 <= y2)) then
    return true
  end

  --right
  if ((x2 <= x3 and x2 >= x4) and (y1 >= y3 and y2 <= y4)) or
    ((x4 <= x1 and x4 >= x2) and (y3 >= y1 and y4 <= y2)) then
    return true
  end

  --bottom
  if ((x1 <= x3 and x2 >= x4) and (y2 >= y3 and y2 <= y4)) or
    ((x3 <= x1 and x4 >= x2) and (y4 >= y1 and y4 <= y2)) then
    return true
  end

  return false
end


--checks for collisions between an
--object and all objects in a res_man
--or all objects in two res_man's

function collide_all(t1, t2)

  --obj and res_man type 1
  if t1.x and t2.data then

    local temp = t2.elements
    local c = 1

    while temp ~= 0 do
      if t2.data[c] then
        temp = temp - 1
          if collision(t1, t2.data[c]) then
            t1:collide()
            t2.data[c]:collide()
          end
      end

      c = c + 1
    end
    return
  end


  --obj and res_man type 2
  if t1.data and t2.x then


    local temp = t1.elements
    local c = 1

    while temp ~= 0 do
      if t1.data[c] then
        temp = temp - 1
          if collision(t2, t1.data[c]) then
            t2:collide()
            t1.data[c]:collide()
          end
      end

      c = c + 1
    end
    return
  end


  --two res_man's
  if t1.data and t2.data then

    local temp = t1.elements
    local c = 1
    

    while temp ~= 0 do
      if t1.data[c] then
        temp = temp - 1
        local temp2 = t2.elements
        local c2 = 1
        while temp2 ~= 0 do
          if t2.data[c2] then
            temp2 = temp2 - 1
            if collision(t1.data[c], t2.data[c2]) then
              t1.data[c]:collide()
              t2.data[c2]:collide()
            end
          end
          c2 = c2 + 1
        end
      end

      c = c + 1
    end


    return
  end


end