--[[
File: main.lua
Author: Brett Lajzer
Date: 5.22.2006

About:
This is the main lua script.
It runs everything and makes the magic happen.
--]]


--load up player, enemies, and such
require "scripts/player"
require "scripts/life_bar"
require "scripts/bullet"
require "scripts/resource_manager"
require "scripts/backgrounds"
require "scripts/enemy"
require "scripts/collision"
require "scripts/scoreboard"
require "scripts/menu"
require "scripts/gameover"
require "scripts/all_enemies"
require "scripts/levels"


--global vars
done = false	--used for the main game loop
fire_rate = 0  --used for limiting fire
ticks = 0  --clock ticks (frames since start of level)

bullets = res_man:new()  --resource manager for bullets
player_bullets = res_man:new() --resource manager for player bullets
enemies = res_man:new()  --resource manager for enemies

level = menu.level  --the level function (runs every tick)
new_level = nil  --don't need to do this, it just shows that the var exists
draw_bg = backgrounds.stars_scrolling --the background to draw
e_step = 0

--turn cursor off since player is effectively the cursor
show_cursor(false)



function gen()
  e_step = e_step + 1
  if e_step == 30 then
    enemies:add(enemy:new({x = 50, y = -50, angle = 40, speed = 7}))
    enemies:add(enemy:new({x = 300, y = -50, angle = 90, speed = 5}))
    enemies:add(enemy:new({x = 550, y = -50, angle = 140, speed = 7}))
    e_step = 0
  end
end

--init fps management
init_fps()
set_fps(30)

--init menu
level(true)

while done ~= true do
  left = false right = false

  --player is centered on mouse cursor
  player.x, player.y, left, right = mouse_state()
  player.x = player.x - player.w/2
  player.y = player.y - player.h/2

  --left mouse button fires
  --hold for auto
  if left and fire_rate >= 3 then 
    player.fire() --put in limiter
    fire_rate = 0
   end
 
  --right mouse button ends game
  if right then
    level = menu.level
    level(true)
    ticks = 0
    scoreboard.score = 0
    player.lives = player.lives_max
  end

  level() -- call the level function

  --update everything
  bullets:update_all()
  player_bullets:update_all()
  enemies:update_all()

  --draw everything
  draw_bg()
  enemies:draw_all()
  player_bullets:draw_all()
  player.draw()
  bullets:draw_all()

  --draw ui stuff
  draw_lives()
  scoreboard.draw()

  --redraw screen
  update_screen()

  --calc collisions
  collide_all(player, bullets)
  collide_all(enemies, player_bullets)

  --end game if player loses
  if player.lives <= -1 then
    player.lives = player.lives_max
    new_level = gameover.level
  end

  --firing rate limiter  
  fire_rate = fire_rate + 1
  if fire_rate > 3 then fire_rate = 3 end

  --update clock ticks
  ticks = ticks + 1

  if new_level then
    level = new_level
    level(true)
    ticks = 0
    new_level = nil
  end

  --delay to maintain proper fps
  delay_fps() --must be last line to work properly
end