--[[
File: resource_manager.lua
Author: Brett Lajzer
Date: 5.24.2006

About:
This is a class designed to allow efficient management of
resources. It's a simple table, but it's handled nicely.

The resource manager has two convenience functions:
update_all() and draw_all(). These call the update()
and draw() functions of all contained items (if possible).
--]]

--resource manager

res_man = { data = {}, elements = 0 }

--constructor
function res_man:new(o)
  o = o or {}
  o.data = {}
  o.elements = 0
  setmetatable(o, self)
  self.__index = self
  return o
end


--adds an element in the first non-nil, numerical position
-- in the data table
function res_man:add(element)
  local open = 1

  --get first open slot
  while self.data[open] do open = open + 1 end

  self.data[open] = element
  self.elements = self.elements + 1
end


--empties the structure
function res_man:empty()
  local c = 1

  while self.elements ~= 0 do
    if self.data[c] then
      self.elements = self.elements - 1
      self.data[c] = nil
    end

    c = c + 1
  end

end


--runs all update() functions
function res_man:update_all()

  local temp = self.elements
  local c = 1

  --cleanup objects that require it
  while temp ~= 0 do
    if self.data[c] then
      temp = temp - 1
      if type(self.data[c]) == "table" and self.data[c].update then
        self.data[c]:update()
      end
    end

    if self.data[c] and self.data[c].collect == true then
      self.data[c] = nil
      self.elements = self.elements - 1
    end

    c = c + 1
  end

end


--runs all draw() functions
function res_man:draw_all()

  local temp = self.elements
  local c = 1

  while temp ~= 0 do
    if self.data[c] then
      temp = temp - 1
      if type(self.data[c]) == "table" and self.data[c].draw then
        self.data[c]:draw()
      end
    end

    c = c + 1
  end

end